import pyaudio
from typing import Optional
import queue

from .base_input_device import BaseInputDevice
from ..models.audio_encoding import AudioEncoding

class MicrophoneInput(BaseInputDevice):

    DEFAULT_SAMPLING_RATE = 44100
    DEFAULT_CHUNK_SIZE = 2048

    def __init__(self, pa: pyaudio.PyAudio, device_info: dict, sampling_rate: int = None, chunk_size: int = DEFAULT_CHUNK_SIZE):
        self.device_info = device_info
        sampling_rate = sampling_rate or (self.device_info.get('defaultSampleRate', self.DEFAULT_SAMPLING_RATE))
        super().__init__(int(sampling_rate), AudioEncoding.LINEAR16, chunk_size)
        self.pa = pa
        self.stream = pa.open(
            format=pyaudio.paInt16,
            channels=1,
            rate=self.sampling_rate,
            input=True,
            frames_per_buffer=self.chunk_size,
            input_device_index=int(self.device_info['index']),
            stream_callback=self._stream_callback
        )
        self.queue = queue.Queue()

    def _stream_callback(self, in_data, *_args):
        self.queue.put_nowait(in_data)
        return (None, pyaudio.paContinue)

    def get_audio(self) -> Optional[bytes]:
        try:
            return self.queue.get_nowait()
        except queue.Empty:
            return None


import pyaudio
from .input_device.microphone_input import MicrophoneInput
from .output_device.speaker_output import SpeakerOutput
import logging

logger = logging.getLogger(__name__)

def _get_device_prompt(device_infos: list[dict]) -> str:
    return """Please select a device:
{}
Choice: """.format(
        "\n".join(f"{index}: {device['name']}" for index, device in enumerate(device_infos)))

def create_microphone_input_and_speaker_output(use_first_available_device=False, mic_sampling_rate=None, speaker_sampling_rate=None) -> tuple[MicrophoneInput, SpeakerOutput]:
    pa = pyaudio.PyAudio()
    num_devices = pa.get_device_count()
    devices = list(map(pa.get_device_info_by_index, range(num_devices)))
    input_device_infos = list(filter(lambda device: device['maxInputChannels'] > 0, devices))
    output_device_infos = list(filter(lambda device: device['maxOutputChannels'] > 0, devices))
    if use_first_available_device:
        input_device_info = input_device_infos[0]
        output_device_info = output_device_infos[0]
    else:
        input_device_info = input_device_infos[int(input(_get_device_prompt(input_device_infos)))]
        output_device_info = output_device_infos[int(input(_get_device_prompt(output_device_infos)))]
    logger.info("Using microphone input device: %s", input_device_info['name'])
    microphone_input = MicrophoneInput(pa, input_device_info, sampling_rate=mic_sampling_rate)
    logger.info("Using speaker output device: %s", output_device_info['name'])
    speaker_output = SpeakerOutput(pa, output_device_info, sampling_rate=speaker_sampling_rate) 
    return microphone_input, speaker_output


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vocode', 'vocode.input_device', 'vocode.models', 'vocode.output_device']

package_data = \
{'': ['*']}

install_requires = \
['pyaudio==0.2.13',
 'pydantic==1.10.5',
 'python-dotenv==0.21.1',
 'typing-extensions==4.5.0',
 'websockets==10.4']

setup_kwargs = {
    'name': 'vocode',
    'version': '0.1.8',
    'description': 'The all-in-one voice SDK',
    'long_description': '# vocode Python SDK\n\n```python\nimport asyncio\nimport signal\n\nfrom vocode.conversation import Conversation\nfrom vocode.helpers import create_microphone_input_and_speaker_output\nfrom vocode.models.transcriber import DeepgramTranscriberConfig\nfrom vocode.models.agent import LLMAgentConfig\nfrom vocode.models.synthesizer import AzureSynthesizerConfig\n\nif __name__ == "__main__":\n    microphone_input, speaker_output = create_microphone_input_and_speaker_output(use_first_available_device=True)\n\n    conversation = Conversation(\n        input_device=microphone_input,\n        output_device=speaker_output,\n        transcriber_config=DeepgramTranscriberConfig.from_input_device(microphone_input),\n        agent_config=LLMAgentConfig(prompt_preamble="The AI is having a pleasant conversation about life."),\n        synthesizer_config=AzureSynthesizerConfig.from_output_device(speaker_output)\n    )\n    signal.signal(signal.SIGINT, lambda _0, _1: conversation.deactivate())\n    asyncio.run(conversation.start())\n```\n',
    'author': 'Ajay Raj',
    'author_email': 'ajay@vocode.dev',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import pyaudio

from .base_output_device import BaseOutputDevice
from ..models.audio_encoding import AudioEncoding

class SpeakerOutput(BaseOutputDevice):

    DEFAULT_SAMPLING_RATE = 44100

    def __init__(self, pa: pyaudio.PyAudio, device_info: dict, sampling_rate: int = None, audio_encoding: AudioEncoding = AudioEncoding.LINEAR16):
        self.device_info = device_info
        sampling_rate = sampling_rate or int(self.device_info.get('defaultSampleRate', self.DEFAULT_SAMPLING_RATE))
        super().__init__(sampling_rate, audio_encoding)
        self.pa = pa
        self.stream = self.pa.open(
            output=True,
            channels=1,
            rate=self.sampling_rate,
            format=pyaudio.paInt16,
            output_device_index=int(self.device_info['index'])
        )

    async def send_async(self, chunk):
        self.stream.write(chunk)

    def terminate(self):
        self.stream.close()
        self.pa.close()